#define PI 3.14159265
    
#include <stdio.h>
#include <stdlib.h>

template<class T> class Array {
public:            // constructors, destructors and such
    virtual ~Array(void)          { delete [] data; }
    Array(int sz = 10)            { init(sz); }
    Array(Array<T> const &other);
    Array<T> const &operator=(Array<T> const &other);
                  // interface            
	int size() const;
    T &operator[](int index);        
private:          // data
	int n;
	T *data;
    // initializer            
	void init(int sz);    
};
    

// funzione privata di inizializzazione di un vettore.
	
template <class T> void Array<T>::init(int sz) {
	if (sz < 1) {        
		fprintf(stderr, "Array: cannot create array of size < 1\n"
                            "       requested: %d\n", sz);  
		exit(1);
    }
	n = sz;        
	data = new T[n];    
}


// Copy Constructor

template <class T> Array<T>::Array(Array<T> const &other) {
	n = other.n;
	data = new T[n];        
	for (register int i = 0; i < n; i++)
            data[i] = other.data[i];    
}    

// Overloading "="

template <class T> Array<T> const &Array<T>::operator=(Array<T> const &other) {
	if (this != &other) {
		delete []data;
        n = other.n;            
		data = new T[n];
		for (register int i = 0; i < n; i++)
			data[i] = other.data[i];        
		}
	return (*this);    
}

template <class T> int Array<T>::size() const {
	return (n);
    }    

// Overloading "[]"

template <class T> T &Array<T>::operator[](int index) {
	if (index < 0 || index >= n) {
		fprintf(stderr, "Array: index out of bounds, must be between"
			" 0 and %d\n, requested was: %d\n", n - 1, index); 
		exit(1);        }
        
	return (data[index]);    
}


int main()
{
    Array<int>  intarr;             

    for (int i = 0; i < intarr.size(); i++)
        intarr[i] = i << 2;
    
    Array<double> doublearr;
    
    for (i = 0; i < doublearr.size(); i++)
        doublearr[i] = PI * (i + 1);
    
    for (i = 0; i < intarr.size(); i++)
        printf("intarr[%d]   : %d\n"
                "doublearr[%d]: %g\n",
                i, intarr[i],
                i, doublearr[i]);
    
    return (0);
}



